/*
   The file contains slightly modified base64 encode algorithm from 
   Bob Trower's b64.c file.  
   http://base64.sourceforge.net/
   */
/*
LICENCE:        Copyright (C) 2001 Alexey Klimkin <klimkin@mcst.ru>
                Copyright (C) 2001 Bob Trower, Trantor Standard Systems Inc.

                Permission is hereby granted, free of charge, to any person
                obtaining a copy of this software and associated
                documentation files (the "Software"), to deal in the
                Software without restriction, including without limitation
                the rights to use, copy, modify, merge, publish, distribute,
                sublicense, and/or sell copies of the Software, and to
                permit persons to whom the Software is furnished to do so,
                subject to the following conditions:

                The above copyright notice and this permission notice shall
                be included in all copies or substantial portions of the
                Software.

                THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
                KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
                WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
                PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
                OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
                OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
                OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
                SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
                */

#include "config.h"
#include "internal.h"

/*
** Translation Table as described in RFC1113
*/
static const char cb64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

/*
** encodeblock
**
** encode 3 8-bit binary bytes as 4 '6-bit' characters
*/
static void 
encodeblock( unsigned char in[3], unsigned char out[4], int len )
{
    out[0] = cb64[ in[0] >> 2 ];
    out[1] = cb64[ ((in[0] & 0x03) << 4) | ((in[1] & 0xf0) >> 4) ];
    out[2] = (unsigned char) (len > 1 ? cb64[ ((in[1] & 0x0f) << 2) | ((in[2] & 0xc0) >> 6) ] : '=');
    out[3] = (unsigned char) (len > 2 ? cb64[ in[2] & 0x3f ] : '=');
}

void 
base64( char *dst, const char *src, int sz )
{
    unsigned char in[3], *out = dst;
    int i, len;

    while (sz > 0)
    {
        len = 0;
        for (i = 0; i < 3; i++, sz--)
        {
            if (sz > 0)
            {
                len++;
                in[i] = src[i];
            } else
                in[i] = 0;
        }
        src += 3;
        if (len)
        {
            encodeblock(in, out, len);
            out += 4;
        }
    }
    *out = '\0';
}

#ifdef TEST
int
main(int argc, char *argv[])
{
    char buffer[BUFF_LEN];

    if (argc != 2)
    {
        printf("Usage: %s <string>\n", argv[0]);
        exit(1);
    }
    
    base64(buffer, argv[1], strlen(argv[1]));
    printf("%s\n", buffer);
    return 0;
}

#endif
