/*
 *  This file is
 *    Copyright (C) 2001 Alexey Klimkin <klimkin@mcst.ru>
 *
 *  It is part of httppc, which is
 *    Copyright (C) 2001 Alexey Klimkin <klimkin@mcst.ru>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _INTERNAL_H_
#define _INTERNAL_H_

#define PFX             PACKAGE": "
#define DFLT_XDNS       "128.9.176.32"  // isi.edu
#define DFLT_XDNS_NAME  "isi.edu"
#define DFLT_PROXY_PORT 8080
#define BUFF_LEN        256
#define RWBUFF_LEN      2048
#define MAX_NETS        1024
#define HTTP_VERSION    "1.1"
#define CONF_SYS_WIDE   PREFIX"/etc/"PACKAGE"rc"
#define CONF_USER       "."PACKAGE"rc"
#define DEFAULT_TIMEOUT -1

#define MODE_NONE               0x000
#define MODE_VERBOSE            0x001
#define MODE_QUIET              0x002
#define MODE_CONNECT            0x010
#define MODE_GETHOSTBYNAME      0x020
#define MODE_GETHOSTBYNAME_R    0x040
#define MODE_GETHOSTBYADDR      0x080
#define MODE_GETHOSTBYADDR_R    0x100
#define MODE_SOCKET             0x200
#define MODE_GETADDRINFO        0x400

#define VERBOSE         (httppc__mode & MODE_VERBOSE)
#define QUIET           (httppc__mode & MODE_QUIET)

// do reentrant strerror
#if defined (HAVE_STRERROR_R)

# if defined (STRERROR_R_SUSv3)
    // int strerror_r(int errnum, char *buf, size_t n);
#   define STRERROR(r)     (strerror_r(r, strerror_buffer, BUFF_LEN) < 0 ? "Unknown" : strerror_buffer)
# else   // !STRERROR_R_SUSv3
    // char *strerror_r(int errnum, char *buf, size_t n);
#   define STRERROR(r)     strerror_r(r, strerror_buffer, BUFF_LEN)
# endif  // !STRERROR_R_SUSv3

# define STRERROR_V      char strerror_buffer[BUFF_LEN]

// simple strerror
#elif defined (HAVE_STRERROR)
# define STRERROR(r)     strerror(r)
# define STRERROR_V
#else
# define STRERROR(r)     "errno"
# define STRERROR_V
#endif

//#define family          httppc__family_linux
#define family          httppc__family_sun
#define family_linux    httppc__family_linux
#define family_sun      httppc__family_sun
#define protocol        httppc__protocol
#define log             httppc__log
#define logerr          httppc__logerr
#define logerrno        httppc__logerrno
#define log_sockaddr    httppc__log_sockaddr
#define log_addrinfo    httppc__log_addrinfo
#define log_hostinfo    httppc__log_hostinfo
#define log_poll_events httppc__log_poll_events
#define log_fd_stats    httppc__log_fd_stats
#define ipv4n_to_ipv4p  httppc__ipv4n_to_ipv4p
#define ipv6n_to_ipv4p  httppc__ipv6n_to_ipv4p
#define ipn_to_ipv4p    httppc__ipn_to_ipv4p
#define ipn_to_ipv4n    httppc__ipn_to_ipv4n
#define base64          httppc__base64
#define wait_canread    httppc__wait_canread
#define wait_canwrite   httppc__wait_canwrite
#define wait_canrw      httppc__wait_canrw

#define socktype_s(t) (t == SOCK_STREAM ? "SOCK_STREAM" :\
                       t == SOCK_DGRAM ? "SOCK_DGRAM" :\
                       t == SOCK_SEQPACKET ? "SOCK_SEQPACKET" :\
                       t == SOCK_RAW ? "SOCK_RAW" :\
                       t == SOCK_RDM ? "SOCK_RDM" :\
                       "unknown")
#define family_s(f)   (f <= AF_MAX ? family[f] : "unknown")
#define protocol_s(f) (f <= 133 ? protocol[f] : "unknown")

struct hostent;
struct in_addr;
struct in6_addr;
struct sockaddr;
struct addrinfo;

extern int httppc__mode;
extern const char * family_linux[];
extern const char * family_sun[];
extern const char * protocol[];

void log(const char *format, ...);
void logerr(const char *format, ...);
void logerrno(const char *format, ...);
void log_sockaddr(const struct sockaddr *saddr);
void log_addrinfo(const struct addrinfo *ptr);
void log_hostinfo(const struct hostent *hostinfo);
void log_poll_events(short events);
void log_fd_stats(int fd, int stats);
void ipv4n_to_ipv4p(const struct in_addr *saddr, char *dst, int cnt);
#ifdef AF_INET6
void ipv6n_to_ipv4p(const struct in6_addr *saddr6, char *dst, int cnt);
#endif // AF_INET6
int ipn_to_ipv4p(const struct sockaddr *saddr, char *dst, int cnt);
int ipn_to_ipv4n(const struct sockaddr *saddr, struct in_addr *dst);
void base64(char *dst, const char *src, int len);
int wait_canread(int sockfd, int timeout);
int wait_canwrite(int sockfd, int timeout);
int wait_canrw(int sockfd, int timeout);

#endif
