/*
 *  This file is
 *    Copyright (C) 2001 Alexey Klimkin <klimkin@mcst.ru>
 *
 *  It is part of httppc, which is
 *    Copyright (C) 2001 Alexey Klimkin <klimkin@mcst.ru>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <sys/types.h>
#include <adns.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "config.h"

#define BUFF_LEN    1024

static void
failure_errno(const char *what, int errnoval)
{
    fprintf(stderr, "adns failure: %s: errno=%d: %s\n", what, errnoval, strerror(errnoval));
    exit(1);
}

void
do_request(adns_state adns, const char *domain, int type, const char *stype)
{
    adns_query query;
    adns_answer *answer;
    int r;
    char *name;

    printf("Request for %s\n", stype);
    r = adns_submit(adns, domain, type, adns_qf_usevc, 0, &query);
    if (r)
        failure_errno("submit", r);

    adns_checkconsistency(adns, query);
    
    r = adns_wait(adns, &query, &answer, 0);
    if (r)
        failure_errno("wait", r);

    r = adns_rr_info(answer->type, 0, 0, 0, 
                 answer->rrs.bytes, &name);
    if (r)
        failure_errno("wait", r);
    printf("%s\n", adns_strerror(answer->status));
    if (answer->status == adns_s_ok)
    {
        printf("%x, %s, owner: %s, cname: %s\n", *(unsigned *)answer->rrs.inaddr, name,
               answer->owner?answer->owner:"", answer->cname?answer->cname:"");
        free(name);
    }
}

int
main(int argc, char *argv[])
{
    adns_state adns;
    int r;

//    res_state statp = (res_state)malloc(sizeof(*statp));
    if (argc != 2)
    {
        printf("%s <host>\n", argv[0]);
        exit(1);
    }
    
    r = adns_init_strcfg(&adns, (adns_if_debug|adns_if_noautosys), stderr, "nameserver 128.9.176.32\nnameserver 176.16.4.107"); // isi.edu
//    r = adns_init_strcfg(&adns, (adns_if_debug | adns_if_noautosys), stderr, "nameserver 176.16.4.107\nnameserver 128.9.176.32"); // terra.edu
    if (r)
        failure_errno("init", r);

    do_request(adns, argv[1], adns_r_a, "adns_r_a");
    do_request(adns, argv[1], adns_r_ns_raw, "adns_r_ns_raw");
    do_request(adns, argv[1], adns_r_ns, "adns_r_ns");
    do_request(adns, argv[1], adns_r_cname, "adns_r_cname");
    do_request(adns, argv[1], adns_r_soa_raw, "adns_r_soa_raw");
    do_request(adns, argv[1], adns_r_soa, "adns_r_soa");
    do_request(adns, argv[1], adns_r_ptr_raw, "adns_r_ptr_raw");
    do_request(adns, argv[1], adns_r_ptr, "adns_r_ptr");
    do_request(adns, argv[1], adns_r_hinfo, "adns_r_hinfo");
    do_request(adns, argv[1], adns_r_mx_raw, "adns_r_mx_raw");
    do_request(adns, argv[1], adns_r_mx, "adns_r_mx");
    do_request(adns, argv[1], adns_r_txt, "adns_r_txt");
    do_request(adns, argv[1], adns_r_rp_raw, "adns_r_rp_raw");
    do_request(adns, argv[1], adns_r_rp, "adns_r_rp");
    do_request(adns, argv[1], adns_r_addr, "adns_r_addr");

    adns_finish(adns);
    return 0;
}
