/*
 *  This file is
 *    Copyright (C) 2001 Alexey Klimkin <klimkin@mcst.ru>
 *
 *  It is part of httppc, which is
 *    Copyright (C) 2001 Alexey Klimkin <klimkin@mcst.ru>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <sys/types.h>
#include <netinet/in.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/poll.h>   // poll(), POLLIN, POLLOUT
#include <fcntl.h>
#include "config.h"
#include "sysproto.h"

int
main(int argc, char *argv[])
{
    int sock;
    struct sockaddr_in addr;
    struct hostent *hostinfo;
    int blocking = 0;
    int res;

    if (argc != 4)
    {
        printf("%s <host> <port> <blocking:1|0>\n", argv[0]);
        exit(1);
    }
    blocking = atoi(argv[3]);
    
    sock = socket(AF_INET, SOCK_STREAM, 6);
    if (socket < 0)
    {
        printf("error: %s\n", strerror(errno));
        exit(1);
    }
    if (!blocking)
    {
        int sock_stats;
        sock_stats = fcntl(sock, F_GETFL, 0);
        fcntl(sock, F_SETFL, sock_stats | O_NONBLOCK);
    }

    hostinfo = gethostbyname(argv[1]);
    if (!hostinfo)
    {
        printf("unknown host %s\n", argv[1]);
        exit(1);
    }

    memset(&addr, 0, sizeof(addr));
    addr.sin_family = AF_INET;
    addr.sin_port = htons(atoi(argv[2]));
    addr.sin_addr = *(struct in_addr*)hostinfo->h_addr;

#ifdef HAVE_GETHOSTBYADDR_R
    {
        struct hostent buf, *res2;
        char buffer[256];
        int h_err, r;

#if defined(__GLIBC__)
        r = gethostbyaddr_r(&addr, sizeof(addr), AF_INET,
                            &buf, buffer, 256, &res2, &h_err);
        if (r)
#else
        res2 = gethostbyaddr_r(&addr, sizeof(addr), AF_INET,
                              &buf, buffer, 256, &h_err);
        if (!res2)
#endif
        {
            printf("err %s: %s\n", argv[1], hstrerror(h_err));
            exit(1);
        }
    }
#endif // HAVE_GETHOSTBYADDR_R

    res = connect(sock, (struct sockaddr*)&addr, sizeof(addr));
    if (res < 0)
    {
        if (errno == EINPROGRESS || errno == EWOULDBLOCK) 
        {
            struct pollfd fds[2];
            printf("nonblocking socket status: %s\n", strerror(errno));

            fds[0].fd = sock;
            fds[0].events = POLLIN;
            fds[0].revents = 0;
            fds[1].fd = sock;
            fds[1].events = POLLOUT;
            fds[1].revents = 0;
            res = poll(fds, 2, -1);
            if (res < 0) 
            {
                printf("error: %s\n", strerror(errno));
                exit(1);
            } else if (res == 0) 
            {
                printf("connect timeout\n");
                exit(1);
            }
        } else {
            printf("error: %s\n", strerror(errno));
            exit(1);
        }
    }

    printf("ok\n");
    close(sock);
    return 0;
}
